@@ -59,6 +59,8 @@
 #include <gdal.h>
 #endif
 
+#include "fontcache.h"
+
 # include <cairo-ft.h>
 /*
 #include <pango/pangocairo.h>
@@ -70,27 +72,21 @@
 #include FT_FREETYPE_H
 */
 
-typedef struct facecacheObj faceCacheObj;
-struct facecacheObj {
+typedef struct cairoFaceCache cairoFaceCache;
+struct cairoFaceCache {
   cairo_font_face_t *face;
   FT_Face ftface;
-  char *path;
-  faceCacheObj *next;
-  cairo_user_data_key_t facekey;
+  cairo_font_options_t *options;
+  cairoFaceCache *next;
 };
 
-int freeFaceCache(faceCacheObj *fc)
-{
-  /* printf("***\nface %s has %d cairo references\n***\n",fc->path,cairo_font_face_get_reference_count(fc->face)); */
+void freeCairoFaceCache(cairoFaceCache *fc) {
   cairo_font_face_destroy(fc->face);
-  FT_Done_Face(fc->ftface);
-  free(fc->path);
-  return MS_SUCCESS;
+  cairo_font_options_destroy(fc->options);
 }
 
 typedef struct {
-  faceCacheObj *facecache;
-  FT_Library library;
+  cairoFaceCache *cairofacecache;
   /* dummy surface and context */
   unsigned char dummydata[4];
   cairo_surface_t *dummysurface;
@@ -102,8 +98,7 @@ void initializeCache(void **vcache)
   cairoCacheData *cache = (cairoCacheData*)malloc(sizeof(cairoCacheData));
   *vcache = cache;
 
-  cache->facecache = NULL;
-  FT_Init_FreeType(&(cache->library));
+  cache->cairofacecache = NULL;
   /* dummy surface and context */
   cache->dummysurface = cairo_image_surface_create_for_data(cache->dummydata, CAIRO_FORMAT_ARGB32, 1,1,4);
   cache->dummycr = cairo_create(cache->dummysurface);
@@ -119,18 +114,17 @@ int cleanupCairo(void *cache)
   if(ccache->dummysurface) {
     cairo_surface_destroy(ccache->dummysurface);
   }
-  if(ccache->facecache) {
-    faceCacheObj *next,*cur;
-    cur = ccache->facecache;
+  if(ccache->cairofacecache) {
+    cairoFaceCache *next,*cur;
+    cur = ccache->cairofacecache;
     do {
       next = cur->next;
-      freeFaceCache(cur);
+      freeCairoFaceCache(cur);
       free(cur);
       cur=next;
     } while(cur);
   }
-  if(ccache->library)
-    FT_Done_FreeType(ccache->library);
+
   free(ccache);
   return MS_SUCCESS;
 }
@@ -166,39 +160,20 @@ int freeImageCairo(imageObj *img)
   return MS_SUCCESS;
 }
 
-
-faceCacheObj *getFontFace(cairoCacheData *cache, const char *font)
-{
-  faceCacheObj *newface = NULL;
-  faceCacheObj *cur=cache->facecache;
+static cairoFaceCache* getCairoFontFace(cairoCacheData *cache, FT_Face ftface) {
+  cairoFaceCache *cur = cache->cairofacecache;
   while(cur) {
-    if(!strcmp(cur->path,font))
-      return cur;
+    if(cur->ftface == ftface) return cur;
     cur = cur->next;
   }
-  newface = malloc(sizeof(faceCacheObj));
-
-  if(FT_New_Face(cache->library, font, 0, &(newface->ftface))) {
-    msSetError(MS_RENDERERERR,"Freetype failed to open font %s","getFontFace()",font);
-    free(newface);
-    return NULL;
-  }
-
-  /* Try to select charmap */
-  if (!newface->ftface->charmap) {
-    if( FT_Select_Charmap(newface->ftface, FT_ENCODING_MS_SYMBOL) )
-       FT_Select_Charmap(newface->ftface, FT_ENCODING_APPLE_ROMAN );
-  }
-
-  newface->next = cache->facecache;
-  cache->facecache = newface;
-  newface->face = cairo_ft_font_face_create_for_ft_face(newface->ftface, 0);
-
-  cairo_font_face_set_user_data (newface->face, &newface->facekey,
-                                 &(newface->ftface), (cairo_destroy_func_t) NULL); // we call FT_Done_Face ourselves in freeFaceCache
-
-  newface->path = msStrdup(font);
-  return newface;
+  cur = msSmallMalloc(sizeof(cairoFaceCache));
+  cur->next = cache->cairofacecache;
+  cache->cairofacecache = cur;
+  cur->ftface = ftface;
+  cur->face = cairo_ft_font_face_create_for_ft_face(ftface, 0);
+  cur->options = cairo_font_options_create();
+  cairo_font_options_set_hint_style(cur->options,CAIRO_HINT_STYLE_NONE);
+  return cur;
 }
 
 #define msCairoSetSourceColor(cr, c) cairo_set_source_rgba((cr),(c)->red/255.0,(c)->green/255.0,(c)->blue/255.0,(c)->alpha/255.0);
@@ -418,65 +393,6 @@ int renderSVGSymbolCairo(imageObj *img, double x, double y, symbolObj *symbol,
 #endif
 }
 
-int renderTruetypeSymbolCairo(imageObj *img, double x, double y, symbolObj *symbol,
-                              symbolStyleObj *s)
-{
-  int unicode;
-  cairo_glyph_t glyph;
-  cairo_text_extents_t extents;
-
-  cairo_matrix_t trans;
-  double ox,oy;
-  cairoCacheData *cache = MS_IMAGE_RENDERER_CACHE(img);
-  cairo_renderer *r = CAIRO_RENDERER(img);
-  faceCacheObj *face = getFontFace(cache,symbol->full_font_path);
-
-  if(!face) return MS_FAILURE;
-
-  cairo_save(r->cr);
-  cairo_set_font_face(r->cr,face->face);
-  cairo_set_font_size(r->cr,s->scale*96/72.0);
-
-
-  msUTF8ToUniChar(symbol->character, &unicode);
-
-  if (face->ftface->charmap &&
-    face->ftface->charmap->encoding == FT_ENCODING_MS_SYMBOL)
-    unicode |= 0xf000;
-
-  glyph.index = FT_Get_Char_Index(face->ftface, unicode);
-  glyph.x=0;
-  glyph.y=0;
-  cairo_glyph_extents(r->cr,&glyph,1,&extents);
-  ox=extents.x_bearing+extents.width/2.;
-  oy=extents.y_bearing+extents.height/2.;
-
-
-
-  cairo_matrix_init_rotate(&trans,-s->rotation);
-
-  cairo_matrix_transform_point(&trans,&ox,&oy);
-  /* cairo_translate(cr,-extents.width/2,-extents.height/2); */
-
-  cairo_translate(r->cr,x-ox,y-oy);
-  cairo_rotate(r->cr, -s->rotation);
-
-  cairo_glyph_path(r->cr,&glyph,1);
-
-  if (s->outlinewidth) {
-    msCairoSetSourceColor(r->cr, s->outlinecolor);
-    cairo_set_line_width(r->cr, s->outlinewidth + 1);
-    cairo_stroke_preserve(r->cr);
-  }
-  if(s->color) {
-    msCairoSetSourceColor(r->cr, s->color);
-    cairo_fill_preserve(r->cr);
-  }
-  cairo_new_path(r->cr);
-  cairo_restore(r->cr);
-  return MS_SUCCESS;
-}
-
 int renderTileCairo(imageObj *img, imageObj *tile, double x, double y)
 {
   cairo_renderer *r = CAIRO_RENDERER(img);
@@ -492,189 +408,49 @@ int renderTileCairo(imageObj *img, imageObj *tile, double x, double y)
   return MS_SUCCESS;
 }
 
-#define CAIROLINESPACE 1.33
-
-int getTruetypeTextBBoxCairo(rendererVTableObj *renderer, char **fonts, int numfonts, double size,
-                             char *text, rectObj *rect, double **advances, int bAdjustBaseline)
-{
-  cairoCacheData *cache = MS_RENDERER_CACHE(renderer);
-  faceCacheObj* face = getFontFace(cache,fonts[0]);
-
-  int curfontidx = 0;
-  char *utfptr=text;
-  int i,unicode;
-  unsigned long previdx=0;
-  faceCacheObj* prevface = face;
-  int numglyphs = msGetNumGlyphs(text);
-  cairo_glyph_t glyph;
-  cairo_text_extents_t extents;
-  double px=0,py=0;
-
-  if(face == NULL) {
-    return MS_FAILURE;
-  }
-
-  cairo_set_font_face(cache->dummycr,face->face);
-  cairo_set_font_size(cache->dummycr,size*96/72.0);
-
-  if(advances != NULL) {
-    *advances = (double*)malloc(numglyphs*sizeof(double));
-  }
-
-  for(i=0; i<numglyphs; i++) {
-    utfptr+=msUTF8ToUniChar(utfptr, &unicode);
-    glyph.x=px;
-    glyph.y=py;
-    if(unicode=='\n') {
-      py += ceil(size*CAIROLINESPACE);
-      px = 0;
-      previdx=0;
-      continue;
-    }
-
-    if (curfontidx != 0) {
-      face = getFontFace(cache, fonts[0]);
-      cairo_set_font_face(cache->dummycr, face->face);
-      curfontidx = 0;
-    }
-
-    if (face->ftface->charmap &&
-      face->ftface->charmap->encoding == FT_ENCODING_MS_SYMBOL)
-      unicode |= 0xf000;
-
-    glyph.index = FT_Get_Char_Index(face->ftface, unicode);
-
-    if (glyph.index == 0) {
-      int j;
-      for (j = 1; j < numfonts; j++) {
-        curfontidx = j;
-        face = getFontFace(cache, fonts[j]);
-        glyph.index = FT_Get_Char_Index(face->ftface, unicode);
-        if (glyph.index != 0) {
-          cairo_set_font_face(cache->dummycr, face->face);
-          break;
-        }
-      }
-    }
-
-    if( FT_HAS_KERNING((prevface->ftface)) && previdx ) {
-      FT_Vector delta;
-      FT_Get_Kerning( prevface->ftface, previdx, glyph.index, FT_KERNING_DEFAULT, &delta );
-      px += delta.x / 64.;
-    }
-    cairo_glyph_extents(cache->dummycr,&glyph,1,&extents);
-
-    if(i==0) {
-      rect->minx = px+extents.x_bearing;
-      rect->miny = py+extents.y_bearing;
-      rect->maxx = px+extents.x_bearing+extents.width;
-      rect->maxy = py+(bAdjustBaseline?1:(extents.y_bearing+extents.height));
-    } else {
-      rect->minx = MS_MIN(rect->minx,px+extents.x_bearing);
-      rect->miny = MS_MIN(rect->miny,py+extents.y_bearing);
-      rect->maxy = MS_MAX(rect->maxy,py+(bAdjustBaseline?1:(extents.y_bearing+extents.height)));
-      rect->maxx = MS_MAX(rect->maxx,px+extents.x_bearing+extents.width);
-    }
-    if(advances!=NULL)
-      (*advances)[i]=extents.x_advance;
-    px += extents.x_advance;
-    previdx=glyph.index;
-    prevface = face;
-  }
-  return MS_SUCCESS;
-}
-
-int renderGlyphsCairo(imageObj *img,double x, double y, labelStyleObj *style, char *text)
-{
+int renderGlyphs2Cairo(imageObj *img, textPathObj *tp, colorObj *c, colorObj *oc, int ow) {
   cairo_renderer *r = CAIRO_RENDERER(img);
   cairoCacheData *cache = MS_IMAGE_RENDERER_CACHE(img);
-  faceCacheObj* face = getFontFace(cache,style->fonts[0]);
+  cairoFaceCache *cairo_face = NULL;
+  FT_Face prevface = NULL;
+  int g;
 
-  int curfontidx = 0;
-  char *utfptr=text;
-  int i,unicode;
-  unsigned long previdx=0;
-  faceCacheObj* prevface = face;
-  int numglyphs = msGetNumGlyphs(text);
-  cairo_glyph_t glyph;
-  cairo_text_extents_t extents;
-  double px=0,py=0;
-
-  if(face == NULL) {
-    return MS_FAILURE;
-  }
-
-  cairo_set_font_face(r->cr,face->face);
-  cairo_set_font_size(r->cr,style->size*96/72.0);
-
-  cairo_save(r->cr);
-  cairo_translate(r->cr,MS_NINT(x),MS_NINT(y));
-  if(style->rotation != 0.0)
-    cairo_rotate(r->cr, -style->rotation);
-
-  for(i=0; i<numglyphs; i++) {
-    utfptr+=msUTF8ToUniChar(utfptr, &unicode);
-    glyph.x=px;
-    glyph.y=py;
-    if(unicode=='\n') {
-      py += ceil(style->size*CAIROLINESPACE);
-      px = 0;
-      previdx=0;
-      continue;
+  cairo_set_font_size(r->cr,MS_NINT(tp->glyph_size * 96.0/72.0));
+  for(g=0;g<tp->numglyphs;g++) {
+    glyphObj *gl = &tp->glyphs[g];
+    cairo_glyph_t glyph;
+    /* load the glyph's face into cairo, if not already present */
+    if(gl->face->face != prevface) {
+      cairo_face = getCairoFontFace(cache,gl->face->face);
+      cairo_set_font_face(r->cr, cairo_face->face);
+      cairo_set_font_options(r->cr,cairo_face->options);
+      prevface = gl->face->face;
+      cairo_set_font_size(r->cr,MS_NINT(tp->glyph_size * 96.0/72.0));
     }
-    if(curfontidx != 0) {
-      face = getFontFace(cache,style->fonts[0]);
-      cairo_set_font_face(r->cr,face->face);
-      curfontidx = 0;
-    }
-
-    if (face->ftface->charmap &&
-      face->ftface->charmap->encoding == FT_ENCODING_MS_SYMBOL)
-      unicode |= 0xf000;
-
-    glyph.index = FT_Get_Char_Index(face->ftface, unicode);
-    if(glyph.index == 0) {
-      int j;
-      for(j=1; j<style->numfonts; j++) {
-        curfontidx = j;
-        face = getFontFace(cache,style->fonts[j]);
-        glyph.index = FT_Get_Char_Index(face->ftface, unicode);
-        if(glyph.index != 0) {
-          cairo_set_font_face(r->cr,face->face);
-          break;
-        }
-      }
-    }
-
-    if(  FT_HAS_KERNING((prevface->ftface)) && previdx ) {
-      FT_Vector delta;
-      FT_Get_Kerning( prevface->ftface, previdx, glyph.index, FT_KERNING_DEFAULT, &delta );
-      px += delta.x / 64.;
-    }
-    cairo_glyph_extents(r->cr,&glyph,1,&extents);
-    cairo_glyph_path(r->cr,&glyph,1);
-    px += extents.x_advance;
-    previdx=glyph.index;
-    prevface=face;
-  }
-
-  if (style->outlinewidth > 0) {
     cairo_save(r->cr);
-    msCairoSetSourceColor(r->cr, style->outlinecolor);
-    cairo_set_line_width(r->cr, style->outlinewidth + 1);
+    cairo_translate(r->cr,gl->pnt.x,gl->pnt.y);
+    if(gl->rot != 0.0)
+      cairo_rotate(r->cr, -gl->rot);
+    glyph.x = glyph.y = 0;
+    glyph.index = gl->glyph->key.codepoint;
+    cairo_glyph_path(r->cr,&glyph,1);
+    cairo_restore(r->cr);
+  }
+  if (oc) {
+    cairo_save(r->cr);
+    msCairoSetSourceColor(r->cr, oc);
+    cairo_set_line_width(r->cr, ow + 1);
     cairo_stroke_preserve(r->cr);
     cairo_restore(r->cr);
   }
-  if(style->color) {
-    msCairoSetSourceColor(r->cr, style->color);
+  if(c) {
+    msCairoSetSourceColor(r->cr, c);
     cairo_fill(r->cr);
   }
   cairo_new_path(r->cr);
-  cairo_restore(r->cr);
   return MS_SUCCESS;
 }
 
-
 cairo_status_t _stream_write_fn(void *b, const unsigned char *data, unsigned int length)
 {
   msBufferAppend((bufferObj*)b,(void*)data,length);
@@ -855,9 +631,7 @@ static void msTransformToGeospatialPDF(imageObj *img, mapObj *map, cairo_rendere
         msBufferResize(r->outputStream, nFileSize);
 
         VSIFSeekL(fp, 0, SEEK_SET);
-        VSIFReadL(r->outputStream->data, 1, nFileSize, fp);
-
-        r->outputStream->size = nFileSize;
+        r->outputStream->size = VSIFReadL(r->outputStream->data, 1, nFileSize, fp);
 
         VSIFCloseL(fp);
         fp = NULL;
@@ -932,7 +706,7 @@ int renderEllipseSymbolCairo(imageObj *img, double x, double y, symbolObj *symbo
 
 int startLayerVectorCairo(imageObj *img, mapObj *map, layerObj *layer)
 {
-  if(layer->opacity<100) {
+  if(layer->compositer && layer->compositer->opacity<100) {
     cairo_renderer *r = CAIRO_RENDERER(img);
     cairo_push_group (r->cr);
   }
@@ -941,10 +715,10 @@ int startLayerVectorCairo(imageObj *img, mapObj *map, layerObj *layer)
 
 int closeLayerVectorCairo(imageObj *img, mapObj *map, layerObj *layer)
 {
-  if(layer->opacity<100) {
+  if(layer->compositer && layer->compositer->opacity<100) {
     cairo_renderer *r = CAIRO_RENDERER(img);
     cairo_pop_group_to_source (r->cr);
-    cairo_paint_with_alpha (r->cr, layer->opacity*0.01);
+    cairo_paint_with_alpha (r->cr, layer->compositer->opacity*0.01);
   }
   return MS_SUCCESS;
 }
@@ -990,7 +764,7 @@ int getRasterBufferCopyCairo(imageObj *img, rasterBufferObj *rb)
   rb->data.rgba.pixel_step=4;
   rb->width = cairo_image_surface_get_width(r->surface);
   rb->height = cairo_image_surface_get_height(r->surface);
-  pb = (unsigned char*)malloc(rb->height * rb->data.rgba.row_step * sizeof(unsigned char*));
+  pb = (unsigned char*)malloc(rb->height * rb->data.rgba.row_step * sizeof(unsigned char));
   memcpy(pb,cairo_image_surface_get_data(r->surface),rb->height * rb->data.rgba.row_step);
   rb->data.rgba.pixels = pb;
   rb->data.rgba.r = &(pb[2]);
@@ -1003,7 +777,85 @@ int getRasterBufferCopyCairo(imageObj *img, rasterBufferObj *rb)
   return MS_SUCCESS;
 }
 
+static cairo_operator_t ms2cairo_compop(CompositingOperation op) {
+  switch(op) {
+    case MS_COMPOP_CLEAR:
+      return CAIRO_OPERATOR_CLEAR;
+    case MS_COMPOP_SRC:
+      return CAIRO_OPERATOR_SOURCE;
+    case MS_COMPOP_DST:
+      return CAIRO_OPERATOR_DEST;
+    case MS_COMPOP_SRC_OVER:
+      return CAIRO_OPERATOR_OVER;
+    case MS_COMPOP_DST_OVER:
+      return CAIRO_OPERATOR_DEST_OVER;
+    case MS_COMPOP_SRC_IN:
+      return CAIRO_OPERATOR_IN;
+    case MS_COMPOP_DST_IN:
+      return CAIRO_OPERATOR_DEST_IN;
+    case MS_COMPOP_SRC_OUT:
+      return CAIRO_OPERATOR_OUT;
+    case MS_COMPOP_DST_OUT:
+      return CAIRO_OPERATOR_DEST_OUT;
+    case MS_COMPOP_SRC_ATOP:
+      return CAIRO_OPERATOR_ATOP;
+    case MS_COMPOP_DST_ATOP:
+      return CAIRO_OPERATOR_DEST_ATOP;
+    case MS_COMPOP_XOR:
+      return CAIRO_OPERATOR_XOR;
+    case MS_COMPOP_PLUS:
+      return CAIRO_OPERATOR_ADD;
+    case MS_COMPOP_MULTIPLY:
+      return CAIRO_OPERATOR_MULTIPLY;
+    case MS_COMPOP_SCREEN:
+      return CAIRO_OPERATOR_SCREEN;
+    case MS_COMPOP_OVERLAY:
+      return CAIRO_OPERATOR_OVERLAY;
+    case MS_COMPOP_DARKEN:
+      return CAIRO_OPERATOR_DARKEN;
+    case MS_COMPOP_LIGHTEN:
+      return CAIRO_OPERATOR_LIGHTEN;
+    case MS_COMPOP_COLOR_DODGE:
+      return CAIRO_OPERATOR_COLOR_DODGE;
+    case MS_COMPOP_COLOR_BURN:
+      return CAIRO_OPERATOR_COLOR_BURN;
+    case MS_COMPOP_HARD_LIGHT:
+      return CAIRO_OPERATOR_HARD_LIGHT;
+    case MS_COMPOP_SOFT_LIGHT:
+      return CAIRO_OPERATOR_SOFT_LIGHT;
+    case MS_COMPOP_DIFFERENCE:
+      return CAIRO_OPERATOR_DIFFERENCE;
+    case MS_COMPOP_EXCLUSION:
+      return CAIRO_OPERATOR_EXCLUSION;
+    case MS_COMPOP_INVERT:
+    case MS_COMPOP_INVERT_RGB:
+    case MS_COMPOP_MINUS:
+    case MS_COMPOP_CONTRAST:
+    default:
+      return CAIRO_OPERATOR_OVER;
+  }
+}
 
+int cairoCompositeRasterBuffer(imageObj *img, rasterBufferObj *rb, CompositingOperation comp, int opacity) {
+  cairo_surface_t *src;
+  cairo_renderer *r;
+  if(rb->type != MS_BUFFER_BYTE_RGBA) {
+    return MS_FAILURE;
+  }
+  r = CAIRO_RENDERER(img);
+
+  src = cairo_image_surface_create_for_data(rb->data.rgba.pixels,CAIRO_FORMAT_ARGB32,
+        rb->width,rb->height,
+        rb->data.rgba.row_step);
+
+  cairo_set_source_surface (r->cr,src,0,0);
+  cairo_set_operator(r->cr, ms2cairo_compop(comp));
+  cairo_paint_with_alpha(r->cr,opacity/100.0);
+  cairo_surface_finish(src);
+  cairo_surface_destroy(src);
+  cairo_set_operator(r->cr,CAIRO_OPERATOR_OVER);
+  return MS_SUCCESS;
+}
 
 int mergeRasterBufferCairo(imageObj *img, rasterBufferObj *rb, double opacity,
                            int srcX, int srcY, int dstX, int dstY,
@@ -1012,7 +864,7 @@ int mergeRasterBufferCairo(imageObj *img, rasterBufferObj *rb, double opacity,
   cairo_surface_t *src;
   cairo_renderer *r;
   /* not implemented for src,dst,width and height */
-  if(!rb->type == MS_BUFFER_BYTE_RGBA) {
+  if(rb->type != MS_BUFFER_BYTE_RGBA) {
     return MS_FAILURE;
   }
   r = CAIRO_RENDERER(img);
@@ -1038,6 +890,7 @@ int mergeRasterBufferCairo(imageObj *img, rasterBufferObj *rb, double opacity,
 void freeSVGCache(symbolObj *s) {
 #if defined(USE_SVG_CAIRO) || defined(USE_RSVG)
       struct svg_symbol_cache *cache = s->renderer_cache;
+      if(!cache) return;
       assert(cache->svgc);
 #ifdef USE_SVG_CAIRO
       svg_cairo_destroy(cache->svgc);
@@ -1135,6 +988,7 @@ int msPreloadSVGSymbol(symbolObj *symbol)
     cache->svgc = rsvg_handle_new_from_file(symbol->full_pixmap_path,NULL);
     if(!cache->svgc) {
       msSetError(MS_RENDERERERR,"failed to load svg file %s", "msPreloadSVGSymbol()", symbol->full_pixmap_path);
+      return MS_FAILURE;
     }
     rsvg_handle_get_dimensions_sub (cache->svgc, &dim, NULL);
     symbol->sizex = dim.width;
@@ -1163,11 +1017,6 @@ int msRenderRasterizedSVGSymbol(imageObj *img, double x, double y, symbolObj *sy
   symbolStyleObj pixstyle;
   symbolObj pixsymbol;
 
-  if(img->format->renderer == MS_RENDER_WITH_GD) {
-    msSetError(MS_MISCERR, "GD renderer does not support SVG symbols", "msRenderRasterizedSVGSymbol()");
-    return MS_FAILURE;
-  }
-
   if(MS_SUCCESS != msPreloadSVGSymbol(symbol))
     return MS_FAILURE;
   svg_cache = (struct svg_symbol_cache*) symbol->renderer_cache;
@@ -1194,8 +1043,7 @@ int msRenderRasterizedSVGSymbol(imageObj *img, double x, double y, symbolObj *sy
       height = surface_h = symbol->sizey;
     }
     if (style->rotation != 0) {
-      surface_w = MS_NINT(width * 1.415);
-      surface_h = MS_NINT(height * 1.415);
+      surface_w = surface_h = MS_NINT(MS_MAX(height, width) * 1.415);
     }
 
     surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, surface_w, surface_h);
@@ -1235,7 +1083,8 @@ int msRenderRasterizedSVGSymbol(imageObj *img, double x, double y, symbolObj *sy
   pixsymbol.pixmap_buffer = svg_cache->pixmap_buffer;
   pixsymbol.type = MS_SYMBOL_PIXMAP;
 
-  MS_IMAGE_RENDERER(img)->renderPixmapSymbol(img,x,y,&pixsymbol,&pixstyle);
+  if(MS_SUCCESS != MS_IMAGE_RENDERER(img)->renderPixmapSymbol(img,x,y,&pixsymbol,&pixstyle))
+    return MS_FAILURE;
   MS_IMAGE_RENDERER(img)->freeSymbol(&pixsymbol);
   return MS_SUCCESS;
 #else
@@ -1256,7 +1105,7 @@ int msPopulateRendererVTableCairoRaster( rendererVTableObj *renderer )
 {
 #ifdef USE_CAIRO
   renderer->supports_pixel_buffer=1;
-  renderer->supports_transparent_layers = 0;
+  renderer->compositeRasterBuffer = cairoCompositeRasterBuffer;
   renderer->supports_svg = 1;
   renderer->default_transform_mode = MS_TRANSFORM_SIMPLIFY;
   initializeCache(&MS_RENDERER_CACHE(renderer));
@@ -1269,15 +1118,13 @@ int msPopulateRendererVTableCairoRaster( rendererVTableObj *renderer )
   renderer->getRasterBufferHandle=&getRasterBufferHandleCairo;
   renderer->getRasterBufferCopy=&getRasterBufferCopyCairo;
   renderer->renderPolygon=&renderPolygonCairo;
-  renderer->renderGlyphs=&renderGlyphsCairo;
+  renderer->renderGlyphs=&renderGlyphs2Cairo;
   renderer->freeImage=&freeImageCairo;
   renderer->renderEllipseSymbol = &renderEllipseSymbolCairo;
   renderer->renderVectorSymbol = &renderVectorSymbolCairo;
-  renderer->renderTruetypeSymbol = &renderTruetypeSymbolCairo;
   renderer->renderSVGSymbol = &renderSVGSymbolCairo;
   renderer->renderPixmapSymbol = &renderPixmapSymbolCairo;
   renderer->mergeRasterBuffer = &mergeRasterBufferCairo;
-  renderer->getTruetypeTextBBox = &getTruetypeTextBBoxCairo;
   renderer->renderTile = &renderTileCairo;
   renderer->loadImageFromFile = &msLoadMSRasterBufferFromFile;
   renderer->renderPolygonTiled = &renderPolygonTiledCairo;
@@ -1296,7 +1143,7 @@ int populateRendererVTableCairoVector( rendererVTableObj *renderer )
 #ifdef USE_CAIRO
   renderer->use_imagecache=0;
   renderer->supports_pixel_buffer=0;
-  renderer->supports_transparent_layers = 1;
+  renderer->compositeRasterBuffer = NULL;
   renderer->supports_svg = 1;
   renderer->default_transform_mode = MS_TRANSFORM_SIMPLIFY;
   initializeCache(&MS_RENDERER_CACHE(renderer));
@@ -1309,17 +1156,15 @@ int populateRendererVTableCairoVector( rendererVTableObj *renderer )
   renderer->saveImageBuffer = &saveImageBufferCairo;
   renderer->getRasterBufferHandle=&getRasterBufferHandleCairo;
   renderer->renderPolygon=&renderPolygonCairo;
-  renderer->renderGlyphs=&renderGlyphsCairo;
+  renderer->renderGlyphs=&renderGlyphs2Cairo;
   renderer->freeImage=&freeImageCairo;
   renderer->renderEllipseSymbol = &renderEllipseSymbolCairo;
   renderer->renderVectorSymbol = &renderVectorSymbolCairo;
-  renderer->renderTruetypeSymbol = &renderTruetypeSymbolCairo;
   renderer->renderSVGSymbol = &renderSVGSymbolCairo;
   renderer->renderPixmapSymbol = &renderPixmapSymbolCairo;
   renderer->loadImageFromFile = &msLoadMSRasterBufferFromFile;
   renderer->mergeRasterBuffer = &mergeRasterBufferCairo;
   renderer->initializeRasterBuffer = initializeRasterBufferCairo;
-  renderer->getTruetypeTextBBox = &getTruetypeTextBBoxCairo;
   renderer->renderTile = &renderTileCairo;
   renderer->renderPolygonTiled = &renderPolygonTiledCairo;
   renderer->freeSymbol = &freeSymbolCairo;
